@@ -157,24 +157,12 @@ pub struct Variant<T: Form = MetaForm> {
         serde(skip_serializing_if = "Vec::is_empty", default)
     )]
     fields: Vec<Field<T>>,
-    /// Index of the variant, used in `parity-scale-codec`
-    #[cfg_attr(
-        feature = "serde",
-        serde(skip_serializing_if = "Option::is_none", default)
-    )]
-    index: Option<u8>,
-    /// The discriminant of the variant.
+    /// Index of the variant, used in `parity-scale-codec`.
     ///
-    /// # Note
-    ///
-    /// Even though setting the discriminant is optional
-    /// every C-like enum variant has a discriminant specified
-    /// upon compile-time.
-    #[cfg_attr(
-        feature = "serde",
-        serde(skip_serializing_if = "Option::is_none", default)
-    )]
-    discriminant: Option<u64>,
+    /// The value of this will be, in order of precedence:
+    ///     1. The explicit index defined by a `#[codec(index = N)]` attribute.
+    ///     2. The implicit index from the position of the variant in the `enum` definition.
+    index: u8,
     /// Documentation
     #[cfg_attr(
         feature = "serde",
@@ -191,7 +179,6 @@ impl IntoPortable for Variant {
             name: self.name.into_portable(registry),
             fields: registry.map_into_portable(self.fields),
             index: self.index,
-            discriminant: self.discriminant,
             docs: registry.map_into_portable(self.docs),
         }
     }
@@ -202,15 +189,13 @@ impl Variant {
     pub(crate) fn new(
         name: &'static str,
         fields: Vec<Field<MetaForm>>,
-        index: Option<u8>,
-        discriminant: Option<u64>,
+        index: u8,
         docs: Vec<&'static str>,
     ) -> Self {
         Self {
             name,
             fields,
             index,
-            discriminant,
             docs,
         }
     }
@@ -230,16 +215,11 @@ where
         &self.fields
     }
 
-    /// Returns the index of the variant, if specified.
-    pub fn index(&self) -> Option<u8> {
+    /// Returns the index of the variant.
+    pub fn index(&self) -> u8 {
         self.index
     }
 
-    /// Returns the discriminant of the variant.
-    pub fn discriminant(&self) -> Option<u64> {
-        self.discriminant
-    }
-
     /// Returns the documentation of the variant.
     pub fn docs(&self) -> &[T::String] {
         &self.docs
